@@ -142,19 +142,6 @@ static void get_bytes(struct burn_track *track, int count, unsigned char *data)
 	} else valid = 0;
 
 	if (valid <= 0) { /* ts A61031 : extended from (valid == -1) */
-
-		/* ts B00326 */
-		if (!track->eos) { 
-			char msg[80];
-			sprintf(msg, "End of track source after %.f bytes. %s",
-				(double) track->sourcecount,
-				track->open_ended ?
-					"Open ended." : "Fixed size.");
-			libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
-				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
-				msg, 0, 0);
-		}
-
 		track->eos = 1;
 		valid = 0;
 	}
