@@ -350,8 +350,10 @@ struct isoburn_imgen_opts {
     /**
      * Omit the version number (";1") at the end of the ISO-9660 identifiers.
      * Version numbers are usually not used.
+     *      bit0= omit version number with ECMA-119 and Joliet
+     *      bit1= omit version number with Joliet alone
      */
-    unsigned int omit_version_numbers :1;
+    unsigned int omit_version_numbers :2;
 
     /**
      * Allow ISO-9660 directory hierarchy to be deeper than 8 levels.
@@ -375,8 +377,10 @@ struct isoburn_imgen_opts {
      * ISO-9660 forces filenames to have a ".", that separates file name from
      * extension. libisofs adds it if original filename doesn't has one. Set
      * this to 1 to prevent this behavior
+     *      bit0= no forced dot with ECMA-119
+     *      bit1= no forced dot with Joliet
      */
-    unsigned int no_force_dots :1;
+    unsigned int no_force_dots :2;
 
     /**
      * Allow lowercase characters in ISO-9660 filenames. By default, only
@@ -488,6 +492,29 @@ struct isoburn_imgen_opts {
     char scdbackup_tag_time[19];
     char *scdbackup_tag_written;
 
+
+    /* Content of an embedded boot image. Valid if not NULL.
+     * In that case it must point to a memory buffer at least 32 kB.
+     */
+    char *system_area_data;
+    /*
+     * bit0= make bytes 446 - 512 of the system area a partition
+     *       table which reserves partition 1 from byte 63*512 to the
+     *       end of the ISO image. Assumed are 63 secs/hed, 255 head/cyl.
+     *       (GRUB protective msdos label.)
+     *       This works with and without system_area_data.
+     */
+    int system_area_options;
+
+    /* User settable PVD time stamps */
+    time_t vol_creation_time;
+    time_t vol_modification_time;
+    time_t vol_expiration_time;
+    time_t vol_effective_time;
+    /* To eventually override vol_modification_time by unconverted string
+       and timezone 0 */
+    char vol_uuid[17];
+
 };
 
 
