@@ -217,7 +217,7 @@ void isoburn_version(int *major, int *minor, int *micro);
 */
 #define isoburn_libisofs_req_major  0
 #define isoburn_libisofs_req_minor  6
-#define isoburn_libisofs_req_micro 28 
+#define isoburn_libisofs_req_micro 30 
 
 /** The minimum version of libburn to be used with this version of libisoburn
     at compile time.
@@ -261,7 +261,7 @@ int isoburn_libburn_req(int *major, int *minor, int *micro);
 */
 #define isoburn_header_version_major  0
 #define isoburn_header_version_minor  5
-#define isoburn_header_version_micro  2
+#define isoburn_header_version_micro  4
 /** Note:
     Above version numbers are also recorded in configure.ac because libtool
     wants them as parameters at build time.
@@ -962,8 +962,8 @@ int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
     @param relax Bitfield:
            bit0= omit_version_numbers
                  Omit the version number (";1") at the end of the
-                 ISO-9660 identifiers. Version numbers are usually
-                 not used.
+                 ISO-9660 and Joliet identifiers.
+                 Version numbers are usually not used by readers.
            bit1= allow_deep_paths
                  Allow ISO-9660 directory hierarchy to be deeper
                  than 8 levels.
@@ -1012,7 +1012,18 @@ int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
                  prefixed ES fields. This saves 5 to 10 bytes per file and
                  might avoid problems with readers which only accept RRIP.
                  SUSP-1.10 allows it, SUSP-1.12 frowns on it.
-
+          bit12= only_iso_numbers
+                 Same as bit1 omit_version_number but restricted to the names
+                 in the eventual Joliet tree.
+                 @since 0.5.4
+                 For reasons of backward compatibility it is not possible yet
+                 to disable version numbers for ISO 9660 while enabling them
+                 for Joliet. 
+          bit13= no_j_force_dots
+                 Same as no_force_dots but affecting the names in the eventual
+                 Joliet tree rather than the ISO 9660 / ECMA-119 names.
+                 @since 0.5.4
+                 Previous versions added dots to Joliet names unconditionally.
     @return 1 success, <=0 failure
 */
 #define isoburn_igopt_omit_version_numbers       1
@@ -1027,6 +1038,8 @@ int isoburn_igopt_get_extensions(struct isoburn_imgen_opts *o, int *ext);
 #define isoburn_igopt_rrip_version_1_10        512
 #define isoburn_igopt_dir_rec_mtime           1024
 #define isoburn_igopt_aaip_susp_1_10          2048
+#define isoburn_igopt_only_iso_versions       4096
+#define isoburn_igopt_no_j_force_dots         8192
 int isoburn_igopt_set_relaxed(struct isoburn_imgen_opts *o, int relax);
 int isoburn_igopt_get_relaxed(struct isoburn_imgen_opts *o, int *relax);
 
@@ -1167,6 +1180,67 @@ int isoburn_igopt_get_scdbackup_tag(struct isoburn_imgen_opts *o,
                                     char **tag_written);
 
 
+/** Attach 32 kB of binary data which shall get written to the first 32 kB
+    of the ISO image, the System Area.
+    options can cause manipulations of these data before writing happens.
+    If system area data are giveni or options bit0 is set, then bit1 of
+    el_torito_set_isolinux_options() is automatically disabled.
+    @since 0.5.4
+    @param o        The option set to work on
+    @param data     Either NULL or 32 kB of data. Do not submit less bytes !
+    @param options  Can cause manipulations of submitted data before they
+                    get written:
+                    bit0= apply a --protective-msdos-label as of grub-mkisofs.
+                          This means to patch bytes 446 to 512 of the system
+                          area so that one partition is defined which begins
+                          at the second 512-byte block of the image and ends
+                          where the image ends.
+                          This works with and without system_area_data.
+                    bit1= apply isohybrid MBR patching to the system area.
+                          This works only with system area data from
+                          SYSLINUX plus an ISOLINUX boot image (see
+                          iso_image_set_boot_image()) and only if not bit0
+                          is set.
+    @return 1 success, 0 no data to get, <0 failure
+*/
+int isoburn_igopt_set_system_area(struct isoburn_imgen_opts *o,
+                                  char data[32768], int options);
+int isoburn_igopt_get_system_area(struct isoburn_imgen_opts *o,
+                                  char data[32768], int *options);
+
+/** Explicitely set the four timestamps of the emerging ISO image.
+    Default with all parameters is 0.
+    @since 0.5.4
+    ECMA-119 defines the timestamps in the Primary Volume Descriptor as:
+    @param creation_time
+           When "the information in the volume was created."
+           A value of 0 means that the timepoint of write start is to be used.
+    @param modification_time
+           When "the informationin the volume was last modified."
+           A value of 0 means that the timepoint of write start is to be used.
+    @param expiration_time
+           When "the information in the volume may be regarded as obsolete."
+           A value of 0 means that the information never shall expire.
+    @param effective_time
+           When "the information in the volume may be used."
+           A value of 0 means that not such retention is intended.
+    @param uuid
+           If this text is not empty, then it overrides vol_modification_time
+           by copying the first 16 decimal digits from uuid, eventually
+           padding up with decimal '1', and writing a NUL-byte as timezone GMT.
+           It should express a reasonable time in form  YYYYMMDDhhmmsscc
+           E.g.:  2010040711405800 = 7 Apr 2010 11:40:58 (+0 centiseconds)
+    @return 1 success, <=0 failure
+ */
+int isoburn_igopt_set_pvd_times(struct isoburn_imgen_opts *opts,
+                        time_t creation_time, time_t modification_time,
+                        time_t expiration_time, time_t effective_time,
+                        char *uuid);
+int isoburn_igopt_get_pvd_times(struct isoburn_imgen_opts *opts,
+                      time_t *creation_time, time_t *modification_time,
+                      time_t *expiration_time, time_t *effective_time,
+                      char uuid[17]);
+
 /* ----------------------------------------------------------------------- */
 /*                      End of Options for image generation                */
 /* ----------------------------------------------------------------------- */
