@@ -73,6 +73,9 @@ void ecma119_image_free(Ecma119Image *t)
     if (t->output_charset != NULL)
         free(t->output_charset);
 
+    if (t->system_area_data != NULL)
+        free(t->system_area_data);
+
 #ifdef Libisofs_with_checksumS
     if (t->checksum_ctx != NULL) { /* dispose checksum context */
         char md5[16];
@@ -93,7 +96,7 @@ void ecma119_image_free(Ecma119Image *t)
 static
 int need_version_number(Ecma119Image *t, Ecma119Node *n)
 {
-    if (t->omit_version_numbers) {
+    if (t->omit_version_numbers & 1) {
         return 0;
     }
     if (n->type == ECMA119_DIR || n->type == ECMA119_PLACEHOLDER) {
@@ -380,7 +383,7 @@ int ecma119_writer_write_vol_desc(IsoImageWriter *writer)
     IsoImage *image;
     Ecma119Image *t;
     struct ecma119_pri_vol_desc vol;
-
+    int i;
     char *vol_id, *pub_id, *data_id, *volset_id;
     char *system_id, *application_id, *copyright_file_id;
     char *abstract_file_id, *biblio_file_id;
@@ -435,9 +438,44 @@ int ecma119_writer_write_vol_desc(IsoImageWriter *writer)
     strncpy_pad((char*)vol.abstract_file_id, abstract_file_id, 37);
     strncpy_pad((char*)vol.bibliographic_file_id, biblio_file_id, 37);
 
-    iso_datetime_17(vol.vol_creation_time, t->now, t->always_gmt);
-    iso_datetime_17(vol.vol_modification_time, t->now, t->always_gmt);
-    iso_datetime_17(vol.vol_effective_time, t->now, t->always_gmt);
+    if (t->vol_uuid[0]) {
+        for(i = 0; i < 16; i++)
+            if(t->vol_uuid[i] < '0' || t->vol_uuid[i] > '9')
+        break;
+            else
+                vol.vol_creation_time[i] = t->vol_uuid[i];
+       for(; i < 16; i++)
+           vol.vol_creation_time[i] = '1';
+       vol.vol_creation_time[16] = 0;
+    } else if (t->vol_creation_time > 0)
+        iso_datetime_17(vol.vol_creation_time, t->vol_creation_time,
+                        t->always_gmt);
+    else
+        iso_datetime_17(vol.vol_creation_time, t->now, t->always_gmt);
+
+    if (t->vol_uuid[0]) {
+        for(i = 0; i < 16; i++)
+            if(t->vol_uuid[i] < '0' || t->vol_uuid[i] > '9')
+        break;
+            else
+                vol.vol_modification_time[i] = t->vol_uuid[i];
+       for(; i < 16; i++)
+           vol.vol_modification_time[i] = '1';
+       vol.vol_modification_time[16] = 0;
+    } else if (t->vol_modification_time > 0)
+        iso_datetime_17(vol.vol_modification_time, t->vol_modification_time,
+                        t->always_gmt);
+    else
+        iso_datetime_17(vol.vol_modification_time, t->now, t->always_gmt);
+
+    if (t->vol_expiration_time > 0)
+        iso_datetime_17(vol.vol_expiration_time, t->vol_expiration_time,
+                        t->always_gmt);
+
+    if (t->vol_effective_time > 0)
+        iso_datetime_17(vol.vol_effective_time, t->vol_effective_time,
+                        t->always_gmt);
+
     vol.file_structure_version[0] = 1;
 
     free(vol_id);
@@ -1065,7 +1103,9 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
 {
     int ret, i, voldesc_size, nwriters, image_checksums_mad = 0, tag_pos;
     Ecma119Image *target;
-    int el_torito_writer_index = -1, file_src_writer_index= -1;
+    int el_torito_writer_index = -1, file_src_writer_index = -1;
+    int system_area_options = 0;
+    char *system_area = NULL;
 
     /* 1. Allocate target and copy opts there */
     target = calloc(1, sizeof(Ecma119Image));
@@ -1127,6 +1167,30 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->eltorito = (src->bootcat == NULL ? 0 : 1);
     target->catalog = src->bootcat;
 
+    if (opts->system_area_data != NULL) {
+        system_area = opts->system_area_data;
+        system_area_options = opts->system_area_options;
+    } else if (src->system_area_data != NULL) {
+        system_area = src->system_area_data;
+        system_area_options = src->system_area_options;
+    }
+    target->system_area_data = NULL;
+    if (system_area != NULL) {
+        target->system_area_data = calloc(32768, 1);
+        if (target->system_area_data == NULL) {
+            ret = ISO_OUT_OF_MEM;
+            goto target_cleanup;
+        }
+        memcpy(target->system_area_data, system_area, 32768);
+    }
+    target->system_area_options = system_area_options;
+
+    target->vol_creation_time = opts->vol_creation_time;
+    target->vol_modification_time = opts->vol_modification_time;
+    target->vol_expiration_time = opts->vol_expiration_time;
+    target->vol_effective_time = opts->vol_effective_time;
+    strcpy(target->vol_uuid, opts->vol_uuid);
+
     target->input_charset = strdup(iso_get_local_charset(0));
     if (target->input_charset == NULL) {
         ret = ISO_OUT_OF_MEM;
@@ -1689,6 +1753,14 @@ int iso_write_opts_new(IsoWriteOpts **opts, int profile)
     wopts->fifo_size = 1024; /* 2 MB buffer */
     wopts->sort_files = 1; /* file sorting is always good */
 
+    wopts->system_area_data = NULL;
+    wopts->system_area_options = 0;
+    wopts->vol_creation_time = 0;
+    wopts->vol_modification_time = 0;
+    wopts->vol_expiration_time = 0;
+    wopts->vol_effective_time = 0;
+    wopts->vol_uuid[0]= 0;
+
     *opts = wopts;
     return ISO_SUCCESS;
 }
@@ -1700,6 +1772,8 @@ void iso_write_opts_free(IsoWriteOpts *opts)
     }
 
     free(opts->output_charset);
+    if(opts->system_area_data != NULL)
+        free(opts->system_area_data);
     free(opts);
 }
 
@@ -1765,7 +1839,7 @@ int iso_write_opts_set_omit_version_numbers(IsoWriteOpts *opts, int omit)
     if (opts == NULL) {
         return ISO_NULL_POINTER;
     }
-    opts->omit_version_numbers = omit ? 1 : 0;
+    opts->omit_version_numbers = omit & 3;
     return ISO_SUCCESS;
 }
 
@@ -1801,7 +1875,7 @@ int iso_write_opts_set_no_force_dots(IsoWriteOpts *opts, int no)
     if (opts == NULL) {
         return ISO_NULL_POINTER;
     }
-    opts->no_force_dots = no ? 1 : 0;
+    opts->no_force_dots = no & 3;
     return ISO_SUCCESS;
 }
 
@@ -2085,3 +2159,43 @@ int iso_write_opts_get_data_start(IsoWriteOpts *opts, uint32_t *data_start,
     return ISO_SUCCESS;
 }
 
+/*
+ * @param data     Either NULL or 32 kB of data. Do not submit less bytes !
+ * @param options  bit0 = apply GRUB protective msdos label
+ * @param flag     bit0 = invalidate any attached system area data
+ *                        same as data == NULL
+ */
+int iso_write_opts_set_system_area(IsoWriteOpts *opts, char data[32768],
+                                   int options, int flag)
+{
+    if (data == NULL || (flag & 1)) { /* Disable */
+        if (opts->system_area_data != NULL)
+            free(opts->system_area_data);
+        opts->system_area_data = NULL;
+    } else if (!(flag & 2)) {
+        if (opts->system_area_data == NULL) {
+            opts->system_area_data = calloc(32768, 1);
+            if (opts->system_area_data == NULL)
+                return ISO_OUT_OF_MEM;
+        }
+        memcpy(opts->system_area_data, data, 32768);
+    }
+    if (!(flag & 4))
+        opts->system_area_options = options & 3;
+    return ISO_SUCCESS;
+}
+
+int iso_write_opts_set_pvd_times(IsoWriteOpts *opts, 
+                        time_t vol_creation_time, time_t vol_modification_time,
+                        time_t vol_expiration_time, time_t vol_effective_time,
+                        char *vol_uuid)
+{
+    opts->vol_creation_time = vol_creation_time;
+    opts->vol_modification_time = vol_modification_time;
+    opts->vol_expiration_time = vol_expiration_time;
+    opts->vol_effective_time = vol_effective_time;
+    strncpy(opts->vol_uuid, vol_uuid, 16);
+    opts->vol_uuid[16] = 0;
+    return ISO_SUCCESS;
+}
+
