@@ -57,8 +57,10 @@ struct iso_write_opts {
     /**
      * Omit the version number (";1") at the end of the ISO-9660 identifiers.
      * Version numbers are usually not used.
+     * bit0= ECMA-119 and Joliet (for historical reasons)
+     * bit1= Joliet
      */
-    unsigned int omit_version_numbers :1;
+    unsigned int omit_version_numbers :2;
 
     /**
      * Allow ISO-9660 directory hierarchy to be deeper than 8 levels.
@@ -82,8 +84,10 @@ struct iso_write_opts {
      * ISO-9660 forces filenames to have a ".", that separates file name from
      * extension. libisofs adds it if original filename doesn't has one. Set
      * this to 1 to prevent this behavior
+     * bit0= ECMA-119
+     * bit1= Joliet
      */
-    unsigned int no_force_dots :1;
+    unsigned int no_force_dots :2;
 
     /**
      * Allow lowercase characters in ISO-9660 filenames. By default, only
@@ -298,6 +302,22 @@ struct iso_write_opts {
      */
     char *scdbackup_tag_written;
 
+    /*
+     * See ecma119_image : System Area related information
+     */
+    char *system_area_data;
+    int system_area_options;
+
+    /* User settable PVD time stamps */
+    time_t vol_creation_time;
+    time_t vol_modification_time;
+    time_t vol_expiration_time;
+    time_t vol_effective_time;
+    /* To eventually override vol_creation_time and vol_modification_time
+     * by unconverted string with timezone 0
+     */
+    char vol_uuid[17];
+
 };
 
 typedef struct ecma119_image Ecma119Image;
@@ -328,11 +348,11 @@ struct ecma119_image
     unsigned int always_gmt :1;
 
     /* relaxed constraints */
-    unsigned int omit_version_numbers :1;
+    unsigned int omit_version_numbers :2;
     unsigned int allow_deep_paths :1;
     unsigned int allow_longer_paths :1;
     unsigned int max_37_char_filenames :1;
-    unsigned int no_force_dots :1;
+    unsigned int no_force_dots :2;
     unsigned int allow_lowercase :1;
     unsigned int allow_full_ascii :1;
 
@@ -436,6 +456,25 @@ struct ecma119_image
     IsoFileSrc *cat; /**< location of the boot catalog in the new image */
     IsoFileSrc *bootimg; /**< location of the boot image in the new image */
 
+    /*
+     * System Area related information
+     */
+    /* Content of an embedded boot image. Valid if not NULL.
+     * In that case it must point to a memory buffer at least 32 kB.
+     */
+    char *system_area_data;
+    /*
+     * bit0= make bytes 446 - 512 of the system area a partition
+     *       table which reserves partition 1 from byte 63*512 to the
+     *       end of the ISO image. Assumed are 63 secs/hed, 255 head/cyl.
+     *       (GRUB protective msdos label.)
+     *       This works with and without system_area_data.
+     * bit1= apply isohybrid MBR patching to the system area.
+     *       This works only with system_area_data plus ISOLINUX boot image
+     *       and only if not bit0 is set.
+     */
+    int system_area_options;
+
     /*
      * Number of pad blocks that we need to write. Padding blocks are blocks
      * filled by 0s that we put between the directory structures and the file
@@ -492,6 +531,16 @@ struct ecma119_image
     /* writer thread descriptor */
     pthread_t wthread;
     pthread_attr_t th_attr;
+
+    /* User settable PVD time stamps */
+    time_t vol_creation_time;
+    time_t vol_modification_time;
+    time_t vol_expiration_time;
+    time_t vol_effective_time;
+    /* To eventually override vol_creation_time and vol_modification_time
+     * by unconverted string with timezone 0
+     */
+    char vol_uuid[17];
 };
 
 #define BP(a,b) [(b) - (a) + 1]
