@@ -16,7 +16,7 @@
 struct XorrisO; 
 
 
-#define Xorriso_program_versioN "0.5.2"
+#define Xorriso_program_versioN "0.5.4"
 #define Xorriso_program_patch_leveL ""
 
 
@@ -1004,6 +1004,10 @@ int Xorriso_option_volid(struct XorrisO *xorriso, char *volid, int flag);
 /* Option -volset_id */
 int Xorriso_option_volset_id(struct XorrisO *xorriso, char *name, int flag);
 
+/* Option -volume_date */
+int Xorriso_option_volume_date(struct XorrisO *xorriso,
+                               char *time_type, char *timestring, int flag);
+
 /* Option -xattr "on"|"off" */
 int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag);
 
