@@ -261,6 +261,7 @@ struct XorrisO { /* the global context of xorriso */
  int keep_boot_image;
  int patch_isolinux_image;
  char boot_image_bin_path[SfileadrL];
+ char boot_image_bin_form[16];
  int boot_image_emul; /* 0=no emulation
                         (1=emulation as hard disk)
                         (2=emulation as floppy)
@@ -268,6 +269,25 @@ struct XorrisO { /* the global context of xorriso */
  char boot_image_cat_path[SfileadrL];
  off_t boot_image_load_size;
  int boot_image_isohybrid; /* 0=off , 1=auto , 2=on , 3=force */
+ char system_area_disk_path[SfileadrL];
+ int system_area_options;             /* bit0= "GRUB protective msdos label"
+                                               (a simple partition table)
+                                         bit1= isohybrid boot image pointer
+                                               and partition table
+                                       */
+ int patch_system_area;               /* Bits as of system_area_options.
+                                         to be applied to the loaded system
+                                         area of the image, if no 
+                                         system_area_disk_path is set.
+                                       */
+ /* User settable PVD time stamps */
+ time_t vol_creation_time;
+ time_t vol_modification_time;
+ time_t vol_expiration_time;
+ time_t vol_effective_time;
+ /* To eventually override vol_modification_time by unconverted string
+    and timezone 0 */
+ char vol_uuid[17];
 
  /* LBA of boot image after image loading */
  int loaded_boot_bin_lba;
@@ -711,6 +731,9 @@ int Xorriso_append_scdbackup_record(struct XorrisO *xorriso, int flag);
 
 int Xorriso_may_burn(struct XorrisO *xorriso, int flag);
 
+int Xorriso_afile_fopen(struct XorrisO *xorriso,
+                        char *filename, char *mode, FILE **ret_fp, int flag);
+
 
 int Sfile_str(char target[SfileadrL], char *source, int flag);
 
@@ -988,6 +1011,8 @@ struct FindjoB {
    34= get_md5
    35= check_md5
    36= make_md5
+   37= mkisofs_r
+   38= sort_weight number
  */
  int action;
  int prune;
@@ -998,7 +1023,7 @@ struct FindjoB {
  uid_t user;
  gid_t group;
  mode_t mode_and, mode_or;
- int type; /* see Xorriso_set_time flag */
+ int type; /* see Xorriso_set_time flag, also used as weight */
  time_t date;
  char *found_path;
  struct FindjoB *subjob;
